@@ -4,18 +4,21 @@
 # 1) Install a B2G build with Marionette enabled
 # 2) adb forward tcp:2828 tcp:2828
 
+import argparse
 import json
-from optparse import OptionParser
 import os
 import pkg_resources
+import posixpath
 import re
 import shutil
+import sys
 import tempfile
 from zipfile import ZipFile
 
 from marionette import Marionette
 import mozdevice
 import mozlog
+from mozlog import structured
 from gaiatest import GaiaData
 from gaiatest import GaiaDevice
 
@@ -86,26 +89,27 @@ class InvalidCountError(B2GPopulateError):
 
 class B2GPopulate(object):
 
-    PERSISTENT_STORAGE_PATH = '/data/local/storage/persistent'
+    STORAGE_PATH = '/data/local/storage'
 
     handler = mozlog.StreamHandler()
     handler.setFormatter(mozlog.MozFormatter(include_timestamp=True))
     logger = mozlog.getLogger('B2GPopulate', handler)
 
-    def __init__(self, marionette, log_level='INFO', start_timeout=60,
-                 device_serial=None):
+    def __init__(self, marionette, start_timeout=60, device_serial=None):
         self.marionette = marionette
         self.data_layer = GaiaData(self.marionette)
         dm = mozdevice.DeviceManagerADB(deviceSerial=device_serial)
         self.device = GaiaDevice(self.marionette, manager=dm)
 
-        self.logger.setLevel(getattr(mozlog, log_level.upper()))
+        self._logger = structured.get_default_logger(component='b2gpopulate')
+        if not self._logger:
+            self._logger = mozlog.getLogger('b2gpopulate')
         self.start_timeout = start_timeout
 
         if self.device.is_android_build:
             self.idb_dir = 'idb'
             for candidate in self.device.file_manager.list_items(
-                    '/'.join([self.PERSISTENT_STORAGE_PATH, 'chrome'])):
+                    posixpath.join(self.STORAGE_PATH, 'permanent', 'chrome')):
                 if re.match('\d.*idb', candidate):
                     self.idb_dir = candidate
                     break
@@ -118,7 +122,7 @@ class B2GPopulate(object):
             call_count, contact_count, event_count, message_count]])
 
         if restart:
-            self.logger.debug('Stopping B2G')
+            self._logger.debug('Stopping B2G')
             self.device.stop_b2g()
 
         if call_count is not None:
@@ -150,7 +154,7 @@ class B2GPopulate(object):
         db_counts = [WORKLOADS[k]['call'] for k in WORKLOADS.keys()]
         if count not in db_counts:
             raise InvalidCountError('call')
-        self.logger.info('Populating %d calls' % count)
+        self._logger.info('Populating %d calls' % count)
         db_counts.sort(reverse=True)
         for marker in db_counts:
             if count >= marker:
@@ -160,18 +164,18 @@ class B2GPopulate(object):
                 db_zip_name = pkg_resources.resource_filename(
                     __name__, os.path.sep.join(['resources', 'dialerDb.zip']))
                 db_name = 'dialerDb-%d.sqlite' % marker
-                self.logger.debug('Extracting %s from %s' % (
+                self._logger.debug('Extracting %s from %s' % (
                     db_name, db_zip_name))
                 db = ZipFile(db_zip_name).extract(db_name)
                 if restart:
                     self.device.stop_b2g()
-                destination = '/'.join([self.PERSISTENT_STORAGE_PATH,
-                                        '%s+f+app+++%s' % (local_id, key),
-                                        self.idb_dir,
-                                        '2584670174dsitanleecreR.sqlite'])
-                self.logger.debug('Pushing %s to %s' % (db, destination))
+                destination = posixpath.join(self.STORAGE_PATH, 'default',
+                                             '%s+f+app+++%s' % (local_id, key),
+                                             self.idb_dir,
+                                             '2584670174dsitanleecreR.sqlite')
+                self._logger.debug('Pushing %s to %s' % (db, destination))
                 self.device.manager.pushFile(db, destination)
-                self.logger.debug('Removing %s' % db)
+                self._logger.debug('Removing %s' % db)
                 os.remove(db)
                 if restart:
                     self.start_b2g()
@@ -182,10 +186,9 @@ class B2GPopulate(object):
         db_counts = [WORKLOADS[k]['contact'] for k in WORKLOADS.keys()]
         if count not in db_counts:
             raise InvalidCountError('contact')
-        self.device.file_manager.remove('/'.join([
-            self.PERSISTENT_STORAGE_PATH, 'chrome',
-            self.idb_dir, '*csotncta*']))
-        self.logger.info('Populating %d contacts' % count)
+        path = posixpath.join(self.STORAGE_PATH, 'permanent', 'chrome', self.idb_dir)
+        self.device.file_manager.remove(posixpath.join(path, '*csotncta*'))
+        self._logger.info('Populating %d contacts' % count)
         db_counts.sort(reverse=True)
         for marker in db_counts:
             if count >= marker:
@@ -193,33 +196,29 @@ class B2GPopulate(object):
                     __name__, os.path.sep.join(['resources',
                                                 'contactsDb.zip']))
                 db_name = 'contactsDb-%d.sqlite' % marker
-                self.logger.debug('Extracting %s from %s' % (
+                self._logger.debug('Extracting %s from %s' % (
                     db_name, db_zip_name))
                 db = ZipFile(db_zip_name).extract(db_name)
                 if restart:
                     self.device.stop_b2g()
-                destination = '/'.join([
-                    self.PERSISTENT_STORAGE_PATH, 'chrome', self.idb_dir,
-                    '3406066227csotncta.sqlite'])
-                self.logger.debug('Pushing %s to %s' % (db, destination))
+                destination = posixpath.join(path, '3406066227csotncta.sqlite')
+                self._logger.debug('Pushing %s to %s' % (db, destination))
                 self.device.manager.pushFile(db, destination)
-                self.logger.debug('Removing %s' % db)
+                self._logger.debug('Removing %s' % db)
                 os.remove(db)
                 if marker > 0 and include_pictures:
-                    self.logger.debug('Adding contact pictures')
+                    self._logger.debug('Adding contact pictures')
                     pictures_zip_name = pkg_resources.resource_filename(
                         __name__, os.path.sep.join(
                             ['resources', 'contactsPictures.zip']))
                     temp = tempfile.mkdtemp()
-                    self.logger.debug('Extracting %s to %s' % (
+                    self._logger.debug('Extracting %s to %s' % (
                         pictures_zip_name, temp))
                     ZipFile(pictures_zip_name).extractall(temp)
-                    destination = '/'.join([
-                        self.PERSISTENT_STORAGE_PATH, 'chrome', self.idb_dir,
-                        '3406066227csotncta.files'])
-                    self.logger.debug('Pushing %s to %s' % (temp, destination))
+                    destination = posixpath.join(path, '3406066227csotncta.files')
+                    self._logger.debug('Pushing %s to %s' % (temp, destination))
                     self.device.manager.pushDir(temp, destination)
-                    self.logger.debug('Removing %s' % temp)
+                    self._logger.debug('Removing %s' % temp)
                     shutil.rmtree(temp)
                 if restart:
                     self.start_b2g()
@@ -230,7 +229,7 @@ class B2GPopulate(object):
         db_counts = [WORKLOADS[k]['event'] for k in WORKLOADS.keys()]
         if count not in db_counts:
             raise InvalidCountError('event')
-        self.logger.info('Populating %d events' % count)
+        self._logger.info('Populating %d events' % count)
         db_counts.sort(reverse=True)
         for marker in db_counts:
             if count >= marker:
@@ -241,18 +240,18 @@ class B2GPopulate(object):
                     __name__, os.path.sep.join(['resources',
                                                 'calendarDb.zip']))
                 db_name = 'calendarDb-%d.sqlite' % marker
-                self.logger.debug('Extracting %s from %s' % (
+                self._logger.debug('Extracting %s from %s' % (
                     db_name, db_zip_name))
                 db = ZipFile(db_zip_name).extract(db_name)
                 if restart:
                     self.device.stop_b2g()
-                destination = '/'.join([self.PERSISTENT_STORAGE_PATH,
-                                        '%s+f+app+++%s' % (local_id, key),
-                                        self.idb_dir,
-                                        '125582036br2agd-nceal.sqlite'])
-                self.logger.debug('Pushing %s to %s' % (db, destination))
+                destination = posixpath.join(self.STORAGE_PATH, 'default',
+                                             '%s+f+app+++%s' % (local_id, key),
+                                             self.idb_dir,
+                                             '125582036br2agd-nceal.sqlite')
+                self._logger.debug('Pushing %s to %s' % (db, destination))
                 self.device.manager.pushFile(db, destination)
-                self.logger.debug('Removing %s' % db)
+                self._logger.debug('Removing %s' % db)
                 os.remove(db)
                 if restart:
                     self.start_b2g()
@@ -263,46 +262,43 @@ class B2GPopulate(object):
         db_counts = [WORKLOADS[k]['message'] for k in WORKLOADS.keys()]
         if count not in db_counts:
             raise InvalidCountError('message')
-        self.logger.info('Populating %d messages' % count)
+        self._logger.info('Populating %d messages' % count)
         db_counts.sort(reverse=True)
         for marker in db_counts:
             if count >= marker:
                 db_zip_name = pkg_resources.resource_filename(
                     __name__, os.path.sep.join(['resources', 'smsDb.zip']))
                 db_name = 'smsDb-%d.sqlite' % marker
-                self.logger.debug('Extracting %s from %s' % (
+                self._logger.debug('Extracting %s from %s' % (
                     db_name, db_zip_name))
                 db = ZipFile(db_zip_name).extract(db_name)
                 if restart:
                     self.device.stop_b2g()
-                destination = '/'.join([
-                    self.PERSISTENT_STORAGE_PATH, 'chrome', self.idb_dir,
-                    '226660312ssm.sqlite'])
-                self.logger.debug('Pushing %s to %s' % (db, destination))
+                path = posixpath.join(self.STORAGE_PATH, 'permanent', 'chrome', self.idb_dir)
+                destination = posixpath.join(path, '226660312ssm.sqlite')
+                self._logger.debug('Pushing %s to %s' % (db, destination))
                 self.device.manager.pushFile(db, destination)
                 os.remove(db)
                 if marker > 0:
-                    self.logger.debug('Adding message attachments')
+                    self._logger.debug('Adding message attachments')
                     all_attachments_zip_name = pkg_resources.resource_filename(
                         __name__, os.path.sep.join(
                             ['resources', 'smsAttachments.zip']))
                     attachments_zip_name = 'smsAttachments-%d.zip' % marker
-                    self.logger.debug('Extracting %s from %s' % (
+                    self._logger.debug('Extracting %s from %s' % (
                         attachments_zip_name, all_attachments_zip_name))
                     attachments_zip = ZipFile(
                         all_attachments_zip_name).extract(attachments_zip_name)
                     temp = tempfile.mkdtemp()
-                    self.logger.debug('Extracting %s to %s' % (
+                    self._logger.debug('Extracting %s to %s' % (
                         attachments_zip, temp))
                     ZipFile(attachments_zip).extractall(temp)
-                    destination = '/'.join([
-                        self.PERSISTENT_STORAGE_PATH, 'chrome', self.idb_dir,
-                        '226660312ssm.files'])
-                    self.logger.debug('Pushing %s to %s' % (temp, destination))
+                    destination = posixpath.join(path, '226660312ssm.files')
+                    self._logger.debug('Pushing %s to %s' % (temp, destination))
                     self.device.manager.pushDir(temp, destination)
-                    self.logger.debug('Removing %s' % temp)
+                    self._logger.debug('Removing %s' % temp)
                     shutil.rmtree(temp)
-                    self.logger.debug('Removing %s' % attachments_zip)
+                    self._logger.debug('Removing %s' % attachments_zip)
                     os.remove(attachments_zip)
                 if restart:
                     self.start_b2g()
@@ -320,14 +316,14 @@ class B2GPopulate(object):
 
         # copy the mp3 file into a temp location
         with tempfile.NamedTemporaryFile() as local_copy:
-            self.logger.debug('Creating copy of %s at %s' % (
+            self._logger.debug('Creating copy of %s at %s' % (
                 music_file, local_copy.name))
             local_copy.write(open(music_file).read())
             music_file = local_copy.name
 
             mp3 = EasyID3(music_file)
             album_count = math.ceil(float(count) / tracks_per_album)
-            self.logger.info('Populating %d music files (%d album%s)' % (
+            self._logger.info('Populating %d music files (%d album%s)' % (
                 count, album_count, '' if album_count == 1 else 's'))
 
             for i in range(1, count + 1):
@@ -340,9 +336,9 @@ class B2GPopulate(object):
                 mp3.save()
                 remote_filename = '_%s.'.join(
                     iter(local_filename.split('.'))) % i
-                remote_destination = '/'.join([
-                    self.device.manager.deviceRoot, remote_filename])
-                self.logger.debug('Pushing %s to %s' % (
+                remote_destination = posixpath.join(
+                    self.device.manager.deviceRoot, remote_filename)
+                self._logger.debug('Pushing %s to %s' % (
                     music_file, remote_destination))
                 self.device.manager.pushFile(music_file, remote_destination)
 
@@ -355,14 +351,14 @@ class B2GPopulate(object):
         self.populate_files('video', source, count, destination)
 
     def populate_files(self, file_type, source, count, destination=''):
-        destination = '/'.join([self.device.manager.deviceRoot, destination])
+        destination = posixpath.join(self.device.manager.deviceRoot, destination)
         self.remove_media(file_type)
 
-        self.logger.info('Populating %d %s files' % (count, file_type))
+        self._logger.info('Populating %d %s files' % (count, file_type))
         if count > 0:
             source_file = pkg_resources.resource_filename(
                 __name__, os.path.sep.join(['resources', source]))
-            self.logger.debug('Pushing %d copies of %s to %s' % (
+            self._logger.debug('Pushing %d copies of %s to %s' % (
                 count, source_file, destination))
             self.device.file_manager.push_file(source_file, destination, count)
 
@@ -370,14 +366,14 @@ class B2GPopulate(object):
         if self.device.is_android_build:
             files = getattr(self.data_layer, '%s_files' % file_type) or []
             if len(files) > 0:
-                self.logger.info('Removing %d %s files' % (
+                self._logger.info('Removing %d %s files' % (
                     len(files), file_type))
                 volumes = self.get_volumes()
                 for filename in files:
                     # Get the actual location of the file
-                    parts = filename.strip('/').partition('/')
-                    path = '/'.join([volumes[parts[0]], parts[2]])
-                    self.logger.debug('Removing %s' % path)
+                    parts = filename.strip(posixpath.sep).partition(posixpath.sep)
+                    path = posixpath.join(volumes[parts[0]], parts[2])
+                    self._logger.debug('Removing %s' % path)
                     self.device.file_manager.remove(path)
                 files = getattr(self.data_layer, '%s_files' % file_type) or []
             if not len(files) == 0:
@@ -394,162 +390,143 @@ class B2GPopulate(object):
         return dict([v.split()[start:end] for v in vlist.splitlines()[:-1]])
 
     def start_b2g(self):
-        self.logger.debug('Starting B2G')
+        self._logger.debug('Starting B2G')
         self.device.start_b2g(self.start_timeout)
         self.data_layer = GaiaData(self.marionette)
 
 
 def cli():
-    parser = OptionParser(usage='%prog [options]')
-    parser.add_option(
+    parser = argparse.ArgumentParser(
+        description='Content population tool for Firefox OS')
+    parser.add_argument(
         '--address',
-        action='store',
-        dest='address',
         default='localhost:2828',
-        metavar='str',
-        help='address of marionette server (default: %default)')
-    parser.add_option(
+        help='address of marionette server (default: %(default)s)')
+    parser.add_argument(
         '--device-serial',
-        action='store',
-        dest='device_serial',
-        metavar='str',
+        metavar='SERIAL',
         help='serial identifier of device to target')
-    parser.add_option(
-        '--log-level',
-        action='store',
-        dest='log_level',
-        default='INFO',
-        metavar='str',
-        help='threshold for log output (default: %default)')
-    parser.add_option(
+    parser.add_argument(
         '--start-timeout',
-        action='store',
         type=int,
-        dest='start_timeout',
         default=60,
-        metavar='int',
-        help='b2g start timeout in seconds (default: %default)')
-    parser.add_option(
+        metavar='TIMEOUT',
+        help='b2g start timeout in seconds (default: %(default)s)')
+    parser.add_argument(
         '--calls',
-        action='store',
         type=int,
         dest='call_count',
-        metavar='int',
+        metavar='CALLS',
         help='number of calls to create. must be one of: %s' %
              sorted([WORKLOADS[k]['call'] for k in WORKLOADS.keys()]))
-    parser.add_option(
+    parser.add_argument(
         '--contacts',
-        action='store',
         type=int,
         dest='contact_count',
-        metavar='int',
+        metavar='CONTACTS',
         help='number of contacts to create. must be one of: %s' %
              sorted([WORKLOADS[k]['contact'] for k in WORKLOADS.keys()]))
-    parser.add_option(
+    parser.add_argument(
         '--events',
-        action='store',
         type=int,
         dest='event_count',
-        metavar='int',
+        metavar='EVENTS',
         help='number of events to create. must be one of: %s' %
              sorted([WORKLOADS[k]['event'] for k in WORKLOADS.keys()]))
-    parser.add_option(
+    parser.add_argument(
         '--messages',
-        action='store',
         type=int,
         dest='message_count',
-        metavar='int',
+        metavar='MESSAGES',
         help='number of messages to create. must be one of: %s' %
              sorted([WORKLOADS[k]['message'] for k in WORKLOADS.keys()]))
-    parser.add_option(
+    parser.add_argument(
         '--music',
-        action='store',
         type=int,
         dest='music_count',
-        metavar='int',
+        metavar='MUSIC',
         help='number of music files to create')
-    parser.add_option(
+    parser.add_argument(
         '--pictures',
-        action='store',
         type=int,
         dest='picture_count',
-        metavar='int',
+        metavar='PICTURES',
         help='number of pictures to create')
-    parser.add_option(
+    parser.add_argument(
         '--videos',
-        action='store',
         type=int,
         dest='video_count',
-        metavar='int',
+        metavar='VIDEOS',
         help='number of videos to create')
-    parser.add_option(
+    parser.add_argument(
         '--workload',
-        action='store',
-        dest='workload',
-        metavar='str',
+        metavar='WORKLOAD',
         choices=sorted(WORKLOADS, key=WORKLOADS.__getitem__),
         help='type of workload to create. must be one of: %s' %
              sorted(WORKLOADS, key=WORKLOADS.__getitem__))
+    structured.commandline.add_logging_group(parser)
+
+    args = parser.parse_args()
+    structured.commandline.setup_logging(
+        'b2gpopulate', args, {'mach': sys.stdout})
 
-    options, args = parser.parse_args()
     data_types = WORKLOADS['empty'].keys()
     for data_type in data_types:
-        count = getattr(options, '%s_count' % data_type)
+        count = getattr(args, '%s_count' % data_type)
         if count and not count >= 0:
             parser.print_usage()
             print 'Invalid value for %s count!' % data_type
             parser.exit()
 
-    counts = [getattr(options, '%s_count' % data_type) for
+    counts = [getattr(args, '%s_count' % data_type) for
               data_type in data_types]
     if not len([count for count in counts if count >= 0]) > 0:
-        if options.workload is None:
+        if args.workload is None:
             parser.print_usage()
             print 'Must specify at least one item to populate'
             parser.exit()
     else:
-        if options.workload is not None:
+        if args.workload is not None:
             parser.print_usage()
             print 'Please specify either a workload or individual values'
             parser.exit()
 
     # only allow preset values for calls, messages, contacts, events
     for data_type in ['call', 'message', 'contact', 'event']:
-        count = getattr(options, '%s_count' % data_type)
+        count = getattr(args, '%s_count' % data_type)
         preset_counts = [WORKLOADS[k][data_type] for k in WORKLOADS.keys()]
         if count and count not in preset_counts:
             raise InvalidCountError(data_type)
 
     try:
-        host, port = options.address.split(':')
+        host, port = args.address.split(':')
     except ValueError:
         raise B2GPopulateError('--address must be in the format host:port')
 
     marionette = Marionette(host=host, port=int(port), timeout=180000)
     marionette.start_session()
     b2gpopulate = B2GPopulate(marionette,
-                              log_level=options.log_level,
-                              start_timeout=options.start_timeout,
-                              device_serial=options.device_serial)
+                              start_timeout=args.start_timeout,
+                              device_serial=args.device_serial)
 
-    if options.workload is None:
+    if args.workload is None:
         b2gpopulate.populate(
-            options.call_count,
-            options.contact_count,
-            options.message_count,
-            options.music_count,
-            options.picture_count,
-            options.video_count,
-            options.event_count)
+            args.call_count,
+            args.contact_count,
+            args.message_count,
+            args.music_count,
+            args.picture_count,
+            args.video_count,
+            args.event_count)
     else:
         b2gpopulate.populate(
-            WORKLOADS[options.workload]['call'],
-            WORKLOADS[options.workload]['contact'],
-            WORKLOADS[options.workload]['message'],
-            WORKLOADS[options.workload]['music'],
-            WORKLOADS[options.workload]['picture'],
-            WORKLOADS[options.workload]['video'],
-            WORKLOADS[options.workload]['event'])
+            WORKLOADS[args.workload]['call'],
+            WORKLOADS[args.workload]['contact'],
+            WORKLOADS[args.workload]['message'],
+            WORKLOADS[args.workload]['music'],
+            WORKLOADS[args.workload]['picture'],
+            WORKLOADS[args.workload]['video'],
+            WORKLOADS[args.workload]['event'])
 
 
 if __name__ == '__main__':
